@@ -29,10 +29,13 @@
 #include "main-window.h"
 #include "main.h"
 #include "interface.h"
-#include "add-link-dialog.h"
+#include "add-definition-dialog.h"
 #include "printing.h"
 #include "entry.h"
 #include "storage-manager.h"
+#include "event.h"
+#include "definition.h"
+#include "definition-manager-window.h"
 
 static void almanah_main_window_init (AlmanahMainWindow *self);
 static void almanah_main_window_dispose (GObject *object);
@@ -43,27 +46,28 @@ static void mw_entry_buffer_cursor_position_cb (GObject *object, GParamSpec *psp
 static void mw_entry_buffer_insert_text_cb (GtkTextBuffer *text_buffer, GtkTextIter *start, gchar *text, gint len, AlmanahMainWindow *main_window);
 static void mw_entry_buffer_insert_text_after_cb (GtkTextBuffer *text_buffer, GtkTextIter *start, gchar *text, gint len, AlmanahMainWindow *main_window);
 static void mw_entry_buffer_has_selection_cb (GObject *object, GParamSpec *pspec, AlmanahMainWindow *main_window);
+static void mw_entry_buffer_apply_tag_cb (GtkTextBuffer *buffer, GtkTextTag *tag, GtkTextIter *start_iter, GtkTextIter *end_iter, AlmanahMainWindow *main_window);
 static void mw_bold_toggled_cb (GtkToggleAction *action, AlmanahMainWindow *main_window);
 static void mw_italic_toggled_cb (GtkToggleAction *action, AlmanahMainWindow *main_window);
 static void mw_underline_toggled_cb (GtkToggleAction *action, AlmanahMainWindow *main_window);
+static void mw_events_updated_cb (AlmanahEventManager *event_manager, AlmanahEventFactoryType type_id, AlmanahMainWindow *main_window);
 void mw_calendar_day_selected_cb (GtkCalendar *calendar, AlmanahMainWindow *main_window);
-static void mw_links_selection_changed_cb (GtkTreeSelection *tree_selection, AlmanahMainWindow *main_window);
-static void mw_links_value_data_cb (GtkTreeViewColumn *column, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user_data);
+static void mw_events_selection_changed_cb (GtkTreeSelection *tree_selection, AlmanahMainWindow *main_window);
+static void mw_events_value_data_cb (GtkTreeViewColumn *column, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user_data);
+static void mw_definition_removed_cb (AlmanahStorageManager *storage_manager, const gchar *definition_text, AlmanahMainWindow *main_window);
 
 struct _AlmanahMainWindowPrivate {
 	GtkTextView *entry_view;
 	GtkTextBuffer *entry_buffer;
 	GtkCalendar *calendar;
 	GtkLabel *date_label;
-	GtkButton *add_button;
-	GtkButton *remove_button;
 	GtkButton *view_button;
 	GtkAction *add_action;
 	GtkAction *remove_action;
-	GtkListStore *link_store;
-	GtkTreeSelection *links_selection;
-	GtkTreeViewColumn *link_value_column;
-	GtkCellRendererText *link_value_renderer;
+	GtkListStore *event_store;
+	GtkTreeSelection *events_selection;
+	GtkTreeViewColumn *event_value_column;
+	GtkCellRendererText *event_value_renderer;
 	GtkToggleAction *bold_action;
 	GtkToggleAction *italic_action;
 	GtkToggleAction *underline_action;
@@ -104,10 +108,9 @@ almanah_main_window_dispose (GObject *object)
 {
 	AlmanahMainWindowPrivate *priv = ALMANAH_MAIN_WINDOW (object)->priv;
 
-	if (priv->current_entry != NULL) {
+	if (priv->current_entry != NULL)
 		g_object_unref (priv->current_entry);
-		priv->current_entry = NULL;
-	}
+	priv->current_entry = NULL;
 
 	/* Chain up to the parent class */
 	G_OBJECT_CLASS (almanah_main_window_parent_class)->dispose (object);
@@ -120,16 +123,12 @@ almanah_main_window_new (void)
 	AlmanahMainWindow *main_window;
 	AlmanahMainWindowPrivate *priv;
 	GError *error = NULL;
-#ifdef ENABLE_SPELL_CHECKING
-	GtkSpell *gtkspell;
-	gchar *spelling_language;
-#endif /* ENABLE_SPELL_CHECKING */
 	const gchar *interface_filename = almanah_get_interface_filename ();
 	const gchar *object_names[] = {
-		"dry_main_window",
-		"dry_mw_link_store",
-		"dry_mw_view_button_image",
-		"dry_ui_manager",
+		"almanah_main_window",
+		"almanah_mw_event_store",
+		"almanah_mw_view_button_image",
+		"almanah_ui_manager",
 		NULL
 	};
 
@@ -153,7 +152,7 @@ almanah_main_window_new (void)
 	}
 
 	gtk_builder_set_translation_domain (builder, GETTEXT_PACKAGE);
-	main_window = ALMANAH_MAIN_WINDOW (gtk_builder_get_object (builder, "dry_main_window"));
+	main_window = ALMANAH_MAIN_WINDOW (gtk_builder_get_object (builder, "almanah_main_window"));
 	gtk_builder_connect_signals (builder, main_window);
 
 	if (main_window == NULL) {
@@ -164,60 +163,31 @@ almanah_main_window_new (void)
 	priv = ALMANAH_MAIN_WINDOW (main_window)->priv;
 
 	/* Grab our child widgets */
-	priv->entry_view = GTK_TEXT_VIEW (gtk_builder_get_object (builder, "dry_mw_entry_view"));
+	priv->entry_view = GTK_TEXT_VIEW (gtk_builder_get_object (builder, "almanah_mw_entry_view"));
 	priv->entry_buffer = gtk_text_view_get_buffer (priv->entry_view);
-	priv->calendar = GTK_CALENDAR (gtk_builder_get_object (builder, "dry_mw_calendar"));
-	priv->date_label = GTK_LABEL (gtk_builder_get_object (builder, "dry_mw_date_label"));
-	priv->add_button = GTK_BUTTON (gtk_builder_get_object (builder, "dry_mw_add_button"));
-	priv->remove_button = GTK_BUTTON (gtk_builder_get_object (builder, "dry_mw_remove_button"));
-	priv->view_button = GTK_BUTTON (gtk_builder_get_object (builder, "dry_mw_view_button"));
-	priv->add_action = GTK_ACTION (gtk_builder_get_object (builder, "dry_ui_add_link"));
-	priv->remove_action = GTK_ACTION (gtk_builder_get_object (builder, "dry_ui_remove_link"));
-	priv->link_store = GTK_LIST_STORE (gtk_builder_get_object (builder, "dry_mw_link_store"));
-	priv->links_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (gtk_builder_get_object (builder, "dry_mw_links_tree_view")));
-	priv->link_value_column = GTK_TREE_VIEW_COLUMN (gtk_builder_get_object (builder, "dry_mw_link_value_column"));
-	priv->link_value_renderer = GTK_CELL_RENDERER_TEXT (gtk_builder_get_object (builder, "dry_mw_link_value_renderer"));
-	priv->bold_action = GTK_TOGGLE_ACTION (gtk_builder_get_object (builder, "dry_ui_bold"));;
-	priv->italic_action = GTK_TOGGLE_ACTION (gtk_builder_get_object (builder, "dry_ui_italic"));
-	priv->underline_action = GTK_TOGGLE_ACTION (gtk_builder_get_object (builder, "dry_ui_underline"));
-	priv->cut_action = GTK_ACTION (gtk_builder_get_object (builder, "dry_ui_cut"));
-	priv->copy_action = GTK_ACTION (gtk_builder_get_object (builder, "dry_ui_copy"));
-	priv->delete_action = GTK_ACTION (gtk_builder_get_object (builder, "dry_ui_delete"));
+	priv->calendar = GTK_CALENDAR (gtk_builder_get_object (builder, "almanah_mw_calendar"));
+	priv->date_label = GTK_LABEL (gtk_builder_get_object (builder, "almanah_mw_date_label"));
+	priv->view_button = GTK_BUTTON (gtk_builder_get_object (builder, "almanah_mw_view_button"));
+	priv->add_action = GTK_ACTION (gtk_builder_get_object (builder, "almanah_ui_add_definition"));
+	priv->remove_action = GTK_ACTION (gtk_builder_get_object (builder, "almanah_ui_remove_definition"));
+	priv->event_store = GTK_LIST_STORE (gtk_builder_get_object (builder, "almanah_mw_event_store"));
+	priv->events_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (gtk_builder_get_object (builder, "almanah_mw_events_tree_view")));
+	priv->event_value_column = GTK_TREE_VIEW_COLUMN (gtk_builder_get_object (builder, "almanah_mw_event_value_column"));
+	priv->event_value_renderer = GTK_CELL_RENDERER_TEXT (gtk_builder_get_object (builder, "almanah_mw_event_value_renderer"));
+	priv->bold_action = GTK_TOGGLE_ACTION (gtk_builder_get_object (builder, "almanah_ui_bold"));;
+	priv->italic_action = GTK_TOGGLE_ACTION (gtk_builder_get_object (builder, "almanah_ui_italic"));
+	priv->underline_action = GTK_TOGGLE_ACTION (gtk_builder_get_object (builder, "almanah_ui_underline"));
+	priv->cut_action = GTK_ACTION (gtk_builder_get_object (builder, "almanah_ui_cut"));
+	priv->copy_action = GTK_ACTION (gtk_builder_get_object (builder, "almanah_ui_copy"));
+	priv->delete_action = GTK_ACTION (gtk_builder_get_object (builder, "almanah_ui_delete"));
 
 	/* Set up text formatting */
-	gtk_text_buffer_create_tag (priv->entry_buffer, "bold", 
-				    "weight", PANGO_WEIGHT_BOLD, 
-				    NULL);
-	gtk_text_buffer_create_tag (priv->entry_buffer, "italic",
-				    "style", PANGO_STYLE_ITALIC,
-				    NULL);
-	gtk_text_buffer_create_tag (priv->entry_buffer, "underline",
-				    "underline", PANGO_UNDERLINE_SINGLE,
-				    NULL);
+	almanah_interface_create_text_tags (priv->entry_buffer, TRUE);
 
 #ifdef ENABLE_SPELL_CHECKING
-	/* Set up spell checking */
-	spelling_language = gconf_client_get_string (almanah->gconf_client, "/apps/almanah/spelling_language", NULL);
-
-	/* Make sure it's either NULL or a proper locale specifier */
-	if (spelling_language != NULL && spelling_language[0] == '\0') {
-		g_free (spelling_language);
-		spelling_language = NULL;
-	}
-
-	gtkspell = gtkspell_new_attach (priv->entry_view, spelling_language, &error);
-	g_free (spelling_language);
-
-	if (gtkspell == NULL) {
-		GtkWidget *dialog = gtk_message_dialog_new (NULL,
-							    GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-							    _("Spelling checker could not be initialized"));
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", error->message);
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
-
-		g_error_free (error);
-	}
+	/* Set up spell checking, if it's enabled */
+	if (gconf_client_get_bool (almanah->gconf_client, "/apps/almanah/spell_checking_enabled", NULL) == TRUE)
+		almanah_main_window_enable_spell_checking (main_window, NULL);
 #endif /* ENABLE_SPELL_CHECKING */
 
 	/* Make sure we're notified if the cursor moves position so we can check the tag stack */
@@ -231,26 +201,37 @@ almanah_main_window_new (void)
 	/* Similarly, make sure we're notified when there's a selection so we can change the status of cut/copy/paste actions */
 	g_signal_connect (priv->entry_buffer, "notify::has-selection", G_CALLBACK (mw_entry_buffer_has_selection_cb), main_window);
 
+	/* Get notified of applied tags so we can check if referenced definitions still exist */
+	g_signal_connect_after (priv->entry_buffer, "apply-tag", G_CALLBACK (mw_entry_buffer_apply_tag_cb), main_window);
+
 	/* Connect up the formatting actions */
 	g_signal_connect (priv->bold_action, "toggled", G_CALLBACK (mw_bold_toggled_cb), main_window);
 	g_signal_connect (priv->italic_action, "toggled", G_CALLBACK (mw_italic_toggled_cb), main_window);
 	g_signal_connect (priv->underline_action, "toggled", G_CALLBACK (mw_underline_toggled_cb), main_window);
 
+	/* Notification for event changes */
+	g_signal_connect (almanah->event_manager, "events-updated", G_CALLBACK (mw_events_updated_cb), main_window);
+
+	/* Notification for changes to definitions in the database */
+	g_signal_connect (almanah->storage_manager, "definition-removed", G_CALLBACK (mw_definition_removed_cb), main_window);
+
 	/* Select the current day and month */
 	almanah_calendar_month_changed_cb (priv->calendar, NULL);
 	mw_calendar_day_selected_cb (priv->calendar, main_window);
 
 	/* Set up the treeview */
-	g_signal_connect (priv->links_selection, "changed", G_CALLBACK (mw_links_selection_changed_cb), main_window);
-	gtk_tree_view_column_set_cell_data_func (priv->link_value_column, GTK_CELL_RENDERER (priv->link_value_renderer), mw_links_value_data_cb, NULL, NULL);
+	g_signal_connect (priv->events_selection, "changed", G_CALLBACK (mw_events_selection_changed_cb), main_window);
+	gtk_tree_view_column_set_cell_data_func (priv->event_value_column, GTK_CELL_RENDERER (priv->event_value_renderer), mw_events_value_data_cb, NULL, NULL);
 
 	/* Prettify the UI */
-	almanah_interface_embolden_label (GTK_LABEL (gtk_builder_get_object (builder, "dry_mw_calendar_label")));
-	almanah_interface_embolden_label (GTK_LABEL (gtk_builder_get_object (builder, "dry_mw_attached_links_label")));
+	almanah_interface_embolden_label (GTK_LABEL (gtk_builder_get_object (builder, "almanah_mw_calendar_label")));
+	almanah_interface_embolden_label (GTK_LABEL (gtk_builder_get_object (builder, "almanah_mw_events_label")));
 
 #ifndef ENABLE_ENCRYPTION
+#ifndef ENABLE_SPELL_CHECKING
 	/* Remove the "Preferences" entry from the menu */
-	gtk_action_set_visible (GTK_ACTION (gtk_builder_get_object (builder, "dry_ui_preferences")), FALSE);
+	gtk_action_set_visible (GTK_ACTION (gtk_builder_get_object (builder, "almanah_ui_preferences")), FALSE);
+#endif /* !ENABLE_SPELL_CHECKING */
 #endif /* !ENABLE_ENCRYPTION */
 
 	g_object_unref (builder);
@@ -347,7 +328,6 @@ save_current_entry (AlmanahMainWindow *self)
 	editability = almanah_entry_get_editability (priv->current_entry);
 	entry_exists = almanah_storage_manager_entry_exists (almanah->storage_manager, &date);
 	entry_is_empty = almanah_entry_is_empty (priv->current_entry);
-	editability = almanah_entry_get_editability (priv->current_entry);
 
 	/* Make sure they're editable: don't allow entries in the future to be edited,
 	 * but allow entries in the past to be added or edited, as long as permission is given.
@@ -409,20 +389,14 @@ save_current_entry (AlmanahMainWindow *self)
 	almanah_storage_manager_set_entry (almanah->storage_manager, priv->current_entry);
 
 	/* Mark the day on the calendar if the entry was non-empty (and deleted)
-	 * and update the state of the add link button. */
+	 * and update the state of the add definition action. */
 	if (entry_is_empty == TRUE) {
 		gtk_calendar_unmark_day (priv->calendar, g_date_get_day (&date));
 
-		gtk_widget_set_sensitive (GTK_WIDGET (priv->add_button), FALSE);
-		gtk_action_set_sensitive (priv->add_action, FALSE);
-
-		/* Since the entry is empty, remove all the links from the treeview */
-		gtk_list_store_clear (priv->link_store);
+		/* Since the entry is empty, remove all the events from the treeview */
+		gtk_list_store_clear (priv->event_store);
 	} else {
 		gtk_calendar_mark_day (priv->calendar, g_date_get_day (&date));
-
-		gtk_widget_set_sensitive (GTK_WIDGET (priv->add_button), TRUE);
-		gtk_action_set_sensitive (priv->add_action, TRUE);
 	}
 }
 
@@ -437,68 +411,82 @@ get_selected_date (AlmanahMainWindow *self, GDate *date)
 }
 
 static void
-add_link_to_current_entry (AlmanahMainWindow *self)
+add_definition_to_current_entry (AlmanahMainWindow *self)
 {
-	GtkTreeIter iter;
 	AlmanahMainWindowPrivate *priv = self->priv;
+	GtkTextIter start_iter, end_iter;
+	gchar *text;
 
 	g_assert (priv->entry_buffer != NULL);
 	g_assert (gtk_text_buffer_get_char_count (priv->entry_buffer) != 0);
 
-	/* Ensure that something is selected and its widgets displayed */
-	gtk_widget_show_all (almanah->add_link_dialog);
+	if (gtk_text_buffer_get_selection_bounds (priv->entry_buffer, &start_iter, &end_iter) == FALSE)
+		return;
 
-	if (gtk_dialog_run (GTK_DIALOG (almanah->add_link_dialog)) == GTK_RESPONSE_OK) {
-		GDate date;
-		AlmanahLink *link = almanah_add_link_dialog_get_link (ALMANAH_ADD_LINK_DIALOG (almanah->add_link_dialog));
+	text = gtk_text_buffer_get_text (priv->entry_buffer, &start_iter, &end_iter, FALSE);
 
-		if (link == NULL)
+	/* Create the Add Definition dialogue if it doesn't already exist */
+	if (almanah->add_definition_dialog == NULL)
+		almanah->add_definition_dialog = GTK_WIDGET (almanah_add_definition_dialog_new ());
+
+	/* Ensure that something is selected and its widget's displayed */
+	almanah_add_definition_dialog_set_text (ALMANAH_ADD_DEFINITION_DIALOG (almanah->add_definition_dialog), text);
+	g_free (text);
+	gtk_widget_show_all (almanah->add_definition_dialog);
+
+	if (gtk_dialog_run (GTK_DIALOG (almanah->add_definition_dialog)) == GTK_RESPONSE_OK) {
+		AlmanahDefinition *definition;
+
+		definition = almanah_add_definition_dialog_get_definition (ALMANAH_ADD_DEFINITION_DIALOG (almanah->add_definition_dialog));
+		if (definition == NULL)
 			return;
 
 		/* Add to the DB */
-		get_selected_date (self, &date);
-		almanah_storage_manager_add_entry_link (almanah->storage_manager, &date, link);
+		almanah_storage_manager_add_definition (almanah->storage_manager, definition);
 
-		/* Add to the treeview */
-		gtk_list_store_append (priv->link_store, &iter);
-		gtk_list_store_set (priv->link_store, &iter,
-				    0, almanah_link_get_type_id (link),
-				    1, almanah_link_get_value (link),
-				    2, almanah_link_get_value2 (link),
-				    3, almanah_link_get_icon_name (link),
-				    -1);
-
-		g_object_unref (link);
+		/* Add a GtkTextTag to the GtkTextBuffer to mark the definition */
+		gtk_text_buffer_apply_tag_by_name (priv->entry_buffer, "definition", &start_iter, &end_iter);
+		gtk_text_buffer_set_modified (priv->entry_buffer, TRUE);
 	}
 }
 
 static void
-remove_link_from_current_entry (AlmanahMainWindow *self)
+remove_definition_from_current_entry (AlmanahMainWindow *self)
 {
-	gchar *link_type;
-	GDate date;
-	GtkTreeIter iter;
-	GtkTreeModel *model;
-	GList *links;
+	/* We don't actually remove the definition from the database, since other entries may use it;
+	 * we simply remove the formatting. */
 	AlmanahMainWindowPrivate *priv = self->priv;
+	GtkTextIter start_iter, end_iter;
+	GtkTextTag *tag;
+	GtkTextTagTable *tag_table;
 
-	links = gtk_tree_selection_get_selected_rows (priv->links_selection, &model);
-	get_selected_date (self, &date);
+	/* Find the tag in the tag table */
+	tag_table = gtk_text_buffer_get_tag_table (priv->entry_buffer);
+	tag = gtk_text_tag_table_lookup (tag_table, "definition");
+	g_assert (tag != NULL);
 
-	for (; links != NULL; links = links->next) {
-		gtk_tree_model_get_iter (model, &iter, (GtkTreePath*) links->data);
-		gtk_tree_model_get (model, &iter, 0, &link_type, -1);
+	/* Find the boundaries of the tag (ignoring everything except the position
+	 * of the start iter of any selection the user's made). */
+	gtk_text_buffer_get_selection_bounds (priv->entry_buffer, &start_iter, NULL);
+	end_iter = start_iter;
 
-		/* Remove it from the DB */
-		almanah_storage_manager_remove_entry_link (almanah->storage_manager, &date, link_type);
-
-		/* Remove it from the treeview */
-		gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
-
-		gtk_tree_path_free (links->data);
-		g_free (link_type);
+	if (gtk_text_iter_begins_tag (&start_iter, tag) == TRUE) {
+		/* We're at the start of the tag */
+		gtk_text_iter_forward_to_tag_toggle (&end_iter, tag);
+	} else if (gtk_text_iter_ends_tag (&start_iter, tag) == TRUE) {
+		/* We're at the end of the tag */
+		end_iter = start_iter;
+		gtk_text_iter_backward_to_tag_toggle (&start_iter, tag);
+	} else {
+		/* We're somewhere in the middle of the tag */
+		gtk_text_iter_backward_to_tag_toggle (&start_iter, tag);
+		end_iter = start_iter;
+		gtk_text_iter_forward_to_tag_toggle (&end_iter, tag);
 	}
-	g_list_free (links);
+		
+	/* Remove the tag */
+	gtk_text_buffer_remove_tag (priv->entry_buffer, tag, &start_iter, &end_iter);
+	gtk_text_buffer_set_modified (priv->entry_buffer, TRUE);
 }
 
 void
@@ -515,7 +503,7 @@ mw_entry_buffer_cursor_position_cb (GObject *object, GParamSpec *pspec, AlmanahM
 	AlmanahMainWindowPrivate *priv = main_window->priv;
 	GSList *_tag_list = NULL, *tag_list = NULL;
 	gboolean range_selected = FALSE;
-	gboolean bold_toggled = FALSE, italic_toggled = FALSE, underline_toggled = FALSE;
+	gboolean bold_toggled = FALSE, italic_toggled = FALSE, underline_toggled = FALSE, remove_definition_toggled = FALSE;
 
 	/* Ensure we don't overwrite current formatting options when characters are being typed.
 	 * (Execution of this function will be sandwiched between:
@@ -530,6 +518,9 @@ mw_entry_buffer_cursor_position_cb (GObject *object, GParamSpec *pspec, AlmanahM
 	if (range_selected == FALSE)
 		_tag_list = gtk_text_iter_get_tags (&iter);
 
+	/* We can only have the "add definition" action sensitive if there's a range selected */
+	gtk_action_set_sensitive (priv->add_action, (range_selected == TRUE) ? TRUE : FALSE);
+
 	/* Block signal handlers for the formatting actions while we're executing,
 	 * so formatting doesn't get unwittingly changed. */
 	priv->updating_formatting = TRUE;
@@ -553,7 +544,11 @@ mw_entry_buffer_cursor_position_cb (GObject *object, GParamSpec *pspec, AlmanahM
 			underline_toggled = TRUE;
 		}
 
-		if (action) {
+		if (strcmp (tag_name, "definition") == 0) {
+			/* Deal with definition tags slightly differently --- just toggle the sensitivity of the "remove definition" action */
+			gtk_action_set_sensitive (priv->remove_action, TRUE);
+			remove_definition_toggled = TRUE;
+		} else if (action != NULL) {
 			/* Force the toggle status on the action */
 			gtk_toggle_action_set_active (action, TRUE);
 		} else {
@@ -575,6 +570,8 @@ mw_entry_buffer_cursor_position_cb (GObject *object, GParamSpec *pspec, AlmanahM
 			gtk_toggle_action_set_active (priv->italic_action, FALSE);
 		if (underline_toggled == FALSE)
 			gtk_toggle_action_set_active (priv->underline_action, FALSE);
+		if (remove_definition_toggled == FALSE)
+			gtk_action_set_sensitive (priv->remove_action, FALSE);
 	}
 
 	/* Unblock signals */
@@ -622,6 +619,33 @@ mw_entry_buffer_has_selection_cb (GObject *object, GParamSpec *pspec, AlmanahMai
 	gtk_action_set_sensitive (main_window->priv->delete_action, has_selection);
 }
 
+static void
+mw_entry_buffer_apply_tag_cb (GtkTextBuffer *buffer, GtkTextTag *tag, GtkTextIter *start_iter, GtkTextIter *end_iter, AlmanahMainWindow *main_window)
+{
+	gchar *name;
+
+	g_object_get (G_OBJECT (tag), "name", &name, NULL);
+	if (strcmp (name, "definition") == 0) {
+		AlmanahDefinition *definition;
+		gchar *definition_text;
+
+		/* Check to see if the definition still exists in the DB */
+		definition_text = gtk_text_buffer_get_text (buffer, start_iter, end_iter, FALSE);
+		definition = almanah_storage_manager_get_definition (almanah->storage_manager, definition_text);
+		g_free (definition_text);
+
+		/* If the definition doesn't exist, remove the tag */
+		if (definition == NULL) {
+			gtk_text_buffer_remove_tag (buffer, tag, start_iter, end_iter);
+			gtk_text_buffer_set_modified (buffer, TRUE);
+		} else {
+			g_object_unref (definition);
+		}
+	}
+
+	g_free (name);
+}
+
 static gboolean
 mw_delete_event_cb (GtkWindow *window, gpointer user_data)
 {
@@ -632,10 +656,22 @@ mw_delete_event_cb (GtkWindow *window, gpointer user_data)
 	return TRUE;
 }
 
+void
+mw_page_setup_activate_cb (GtkAction *action, gpointer user_data)
+{
+	almanah_print_page_setup ();
+}
+
+void
+mw_print_preview_activate_cb (GtkAction *action, gpointer user_data)
+{
+	almanah_print_entries (TRUE);
+}
+
 void
 mw_print_activate_cb (GtkAction *action, gpointer user_data)
 {
-	almanah_print_entries ();
+	almanah_print_entries (FALSE);
 }
 
 void
@@ -675,6 +711,9 @@ mw_delete_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
 void
 mw_search_activate_cb (GtkAction *action, gpointer user_data)
 {
+	if (almanah->search_dialog == NULL)
+		almanah->search_dialog = GTK_WIDGET (almanah_search_dialog_new ());
+
 	gtk_widget_show_all (almanah->search_dialog);
 	gtk_dialog_run (GTK_DIALOG (almanah->search_dialog));
 }
@@ -683,6 +722,9 @@ void
 mw_preferences_activate_cb (GtkAction *action, gpointer user_data)
 {
 #ifdef ENABLE_ENCRYPTION
+	if (almanah->preferences_dialog == NULL)
+		almanah->preferences_dialog = GTK_WIDGET (almanah_preferences_dialog_new ());
+
 	gtk_widget_show_all (almanah->preferences_dialog);
 	gtk_dialog_run (GTK_DIALOG (almanah->preferences_dialog));
 #endif /* ENABLE_ENCRYPTION */
@@ -691,18 +733,20 @@ mw_preferences_activate_cb (GtkAction *action, gpointer user_data)
 static void
 apply_formatting (AlmanahMainWindow *self, const gchar *tag_name, gboolean applying)
 {
+	AlmanahMainWindowPrivate *priv = self->priv;
 	GtkTextIter start, end;
 
 	/* Make sure we don't muck up the formatting when the actions are having
 	 * their sensitivity set by the code. */
-	if (self->priv->updating_formatting == TRUE)
+	if (priv->updating_formatting == TRUE)
 		return;
 
-	gtk_text_buffer_get_selection_bounds (self->priv->entry_buffer, &start, &end);
+	gtk_text_buffer_get_selection_bounds (priv->entry_buffer, &start, &end);
 	if (applying == TRUE)
-		gtk_text_buffer_apply_tag_by_name (self->priv->entry_buffer, tag_name, &start, &end);
+		gtk_text_buffer_apply_tag_by_name (priv->entry_buffer, tag_name, &start, &end);
 	else
-		gtk_text_buffer_remove_tag_by_name (self->priv->entry_buffer, tag_name, &start, &end);
+		gtk_text_buffer_remove_tag_by_name (priv->entry_buffer, tag_name, &start, &end);
+	gtk_text_buffer_set_modified (priv->entry_buffer, TRUE);
 }
 
 static void
@@ -727,7 +771,7 @@ void
 mw_about_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
 {
 	gchar *license, *description;
-	guint entry_count, link_count;
+	guint entry_count, definition_count;
 
 	const gchar *authors[] =
 	{
@@ -753,10 +797,10 @@ mw_about_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
 			  _(license_parts[2]),
 			  NULL);
 
-	almanah_storage_manager_get_statistics (almanah->storage_manager, &entry_count, &link_count);
-	description = g_strdup_printf (_("A helpful diary keeper, storing %u entries and %u links."),
+	almanah_storage_manager_get_statistics (almanah->storage_manager, &entry_count, &definition_count);
+	description = g_strdup_printf (_("A helpful diary keeper, storing %u entries and %u definitions."),
 				      entry_count,
-				      link_count);
+				      definition_count);
 
 	gtk_show_about_dialog (GTK_WINDOW (main_window),
 				"version", VERSION,
@@ -789,15 +833,99 @@ mw_jump_to_today_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
 }
 
 void
-mw_add_link_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
+mw_add_definition_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
 {
-	add_link_to_current_entry (main_window);
+	add_definition_to_current_entry (main_window);
 }
 
 void
-mw_remove_link_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
+mw_remove_definition_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
 {
-	remove_link_from_current_entry (main_window);
+	remove_definition_from_current_entry (main_window);
+}
+
+void
+mw_view_definitions_activate_cb (GtkAction *action, AlmanahMainWindow *main_window)
+{
+	if (almanah->definition_manager_window == NULL)
+		almanah->definition_manager_window = GTK_WIDGET (almanah_definition_manager_window_new ());
+
+	gtk_widget_show_all (almanah->definition_manager_window);
+}
+
+static void
+clear_factory_events (AlmanahMainWindow *self, AlmanahEventFactoryType type_id)
+{
+	GtkTreeIter iter;
+	GtkTreeModel *model = GTK_TREE_MODEL (self->priv->event_store);
+
+	if (almanah->debug == TRUE)
+		g_debug ("Removing events belonging to factory %u from the list store...", type_id);
+
+	if (gtk_tree_model_get_iter_first (model, &iter) == FALSE)
+		return;
+
+	while (TRUE) {
+		AlmanahEventFactoryType row_type_id;
+
+		gtk_tree_model_get (model, &iter, 2, &row_type_id, -1);
+
+		if (row_type_id == type_id) {
+			if (almanah->debug == TRUE) {
+				AlmanahEvent *event;
+				gtk_tree_model_get (model, &iter, 0, &event, -1);
+				g_debug ("\t%s", almanah_event_format_value (event));
+			}
+
+			if (gtk_list_store_remove (GTK_LIST_STORE (model), &iter) == FALSE)
+				break;
+		} else if (gtk_tree_model_iter_next (model, &iter) == FALSE) {
+			/* Come to the end of the list */
+			break;
+		}
+	}
+
+	if (almanah->debug == TRUE)
+		g_debug ("Finished removing events.");
+}
+
+static void
+mw_events_updated_cb (AlmanahEventManager *event_manager, AlmanahEventFactoryType type_id, AlmanahMainWindow *main_window)
+{
+	AlmanahMainWindowPrivate *priv = main_window->priv;
+	GSList *_events, *events;
+	GDate date;
+
+	get_selected_date (main_window, &date);
+	_events = almanah_event_manager_get_events (event_manager, type_id, &date);
+
+	/* Clear all the events generated by this factory out of the list store first */
+	clear_factory_events (main_window, type_id);
+
+	if (almanah->debug == TRUE)
+		g_debug ("Adding events from factory %u to the list store...", type_id);
+
+	for (events = _events; events != NULL; events = g_slist_next (events)) {
+		GtkTreeIter iter;
+		AlmanahEvent *event = events->data;
+
+		if (almanah->debug == TRUE)
+			g_debug ("\t%s", almanah_event_format_value (event));
+
+		gtk_list_store_append (priv->event_store, &iter);
+		gtk_list_store_set (priv->event_store, &iter,
+				    0, event,
+				    1, almanah_event_get_icon_name (event),
+				    2, type_id,
+				    -1);
+
+		g_object_unref (event);
+	}
+
+	if (almanah->debug == TRUE)
+		g_debug ("Finished adding events.");
+
+	g_slist_free (_events);
 }
 
 void
@@ -805,9 +933,6 @@ mw_calendar_day_selected_cb (GtkCalendar *calendar, AlmanahMainWindow *main_wind
 {
 	GDate calendar_date;
 	gchar calendar_string[100];
-	AlmanahLink **links;
-	guint i;
-	GtkTreeIter iter;
 #ifdef ENABLE_SPELL_CHECKING
 	GtkSpell *gtkspell;
 #endif /* ENABLE_SPELL_CHECKING */
@@ -832,12 +957,10 @@ mw_calendar_day_selected_cb (GtkCalendar *calendar, AlmanahMainWindow *main_wind
 	gtk_text_buffer_set_modified (priv->entry_buffer, FALSE);
 
 	/* Prepare for the possibility of failure --- do as much of the general interface changes as possible first */
-	gtk_list_store_clear (priv->link_store);
-	gtk_widget_set_sensitive (GTK_WIDGET (priv->remove_button), FALSE); /* Only sensitive if something's selected */
-	gtk_action_set_sensitive (priv->remove_action, FALSE);
-	gtk_widget_set_sensitive (GTK_WIDGET (priv->view_button), FALSE);
-	gtk_widget_set_sensitive (GTK_WIDGET (priv->add_button), FALSE);
+	gtk_list_store_clear (priv->event_store);
 	gtk_action_set_sensitive (priv->add_action, FALSE);
+	gtk_action_set_sensitive (priv->remove_action, FALSE); /* Only sensitive if something's selected */
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->view_button), FALSE);
 
 	if (almanah_entry_is_empty (priv->current_entry) == FALSE) {
 		GError *error = NULL;
@@ -858,9 +981,6 @@ mw_calendar_day_selected_cb (GtkCalendar *calendar, AlmanahMainWindow *main_wind
 
 			return;
 		}
-
-		gtk_widget_set_sensitive (GTK_WIDGET (priv->add_button), TRUE);
-		gtk_action_set_sensitive (priv->add_action, TRUE);
 	} else {
 		/* Set the buffer to be empty */
 		gtk_text_buffer_set_text (priv->entry_buffer, "", -1);
@@ -873,101 +993,46 @@ mw_calendar_day_selected_cb (GtkCalendar *calendar, AlmanahMainWindow *main_wind
 		gtkspell_recheck_all (gtkspell);
 #endif /* ENABLE_SPELL_CHECKING */
 
-	/* List the entry's links */
-	links = almanah_storage_manager_get_entry_links (almanah->storage_manager, &calendar_date);
-
-	i = 0;
-	while (links[i] != NULL) {
-		const gchar *value, *value2;
-
-		value = almanah_link_get_value (links[i]);
-		value2 = almanah_link_get_value2 (links[i]);
-
-		gtk_list_store_append (priv->link_store, &iter);
-		gtk_list_store_set (priv->link_store, &iter,
-				    0, almanah_link_get_type_id (links[i]),
-				    1, value,
-				    2, value2,
-				    3, almanah_link_get_icon_name (links[i]),
-				    -1);
-
-		g_object_unref (links[i]);
-
-		i++;
-	}
-
-	g_free (links);
+	/* List the entry's events */
+	almanah_event_manager_query_events (almanah->event_manager, ALMANAH_EVENT_FACTORY_UNKNOWN, &calendar_date);
 }
 
 static void
-mw_links_selection_changed_cb (GtkTreeSelection *tree_selection, AlmanahMainWindow *main_window)
+mw_events_selection_changed_cb (GtkTreeSelection *tree_selection, AlmanahMainWindow *main_window)
 {
 	AlmanahMainWindowPrivate *priv = main_window->priv;
-
-	if (gtk_tree_selection_count_selected_rows (tree_selection) == 0) {
-		gtk_widget_set_sensitive (GTK_WIDGET (priv->remove_button), FALSE);
-		gtk_widget_set_sensitive (GTK_WIDGET (priv->view_button), FALSE);
-		gtk_action_set_sensitive (priv->remove_action, FALSE);
-	} else {
-		gtk_widget_set_sensitive (GTK_WIDGET (priv->remove_button), TRUE);
-		gtk_widget_set_sensitive (GTK_WIDGET (priv->view_button), TRUE);
-		gtk_action_set_sensitive (priv->remove_action, TRUE);
-	}
+	gtk_widget_set_sensitive (GTK_WIDGET (priv->view_button), (gtk_tree_selection_count_selected_rows (tree_selection) == 0) ? FALSE : TRUE);
 }
 
 static void
-mw_links_value_data_cb (GtkTreeViewColumn *column, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user_data)
+mw_events_value_data_cb (GtkTreeViewColumn *column, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user_data)
 {
-	gchar *new_value, *value, *value2, *type;
-	AlmanahLink *link;
+	const gchar *new_value;
+	AlmanahEvent *event;
 
-	/* TODO: Should really create a new model to render AlmanahLinks --- or at least attach the appropriate AlmanahLink to each tree model row */
+	/* TODO: Should really create a new model to render AlmanahEvents */
 
 	gtk_tree_model_get (model, iter,
-			    0, &type,
-			    1, &value,
-			    2, &value2,
+			    0, &event,
 			    -1);
 
-	link = almanah_link_new (type);
-	almanah_link_set_value (link, value);
-	almanah_link_set_value2 (link, value2);
-
-	new_value = almanah_link_format_value (link);
+	new_value = almanah_event_format_value (event);
 	g_object_set (renderer, "text", new_value, NULL);
-	g_free (new_value);
-
-	g_free (type);
-	g_free (value);
-	g_free (value2);
-	g_object_unref (link);
 }
 
 void
-mw_links_tree_view_row_activated_cb (GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, AlmanahMainWindow *main_window)
+mw_events_tree_view_row_activated_cb (GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, AlmanahMainWindow *main_window)
 {
-	AlmanahLink *link;
-	gchar *value, *value2, *type;
+	AlmanahEvent *event;
 	GtkTreeIter iter;
 
-	gtk_tree_model_get_iter (GTK_TREE_MODEL (main_window->priv->link_store), &iter, path);
-	gtk_tree_model_get (GTK_TREE_MODEL (main_window->priv->link_store), &iter,
-			    0, &type,
-			    1, &value,
-			    2, &value2,
+	gtk_tree_model_get_iter (GTK_TREE_MODEL (main_window->priv->event_store), &iter, path);
+	gtk_tree_model_get (GTK_TREE_MODEL (main_window->priv->event_store), &iter,
+			    0, &event,
 			    -1);
 
-	link = almanah_link_new (type);
-	almanah_link_set_value (link, value);
-	almanah_link_set_value2 (link, value2);
-
-	/* NOTE: Link types should display their own errors, so one won't be displayed here. */
-	almanah_link_view (link);
-
-	g_free (type);
-	g_free (value);
-	g_free (value2);
-	g_object_unref (link);
+	/* NOTE: event types should display their own errors, so one won't be displayed here. */
+	almanah_event_view (event);
 }
 
 gboolean
@@ -977,52 +1042,72 @@ mw_entry_view_focus_out_event_cb (GtkWidget *entry_view, GdkEventFocus *event, A
 	return FALSE;
 }
 
-void
-mw_add_button_clicked_cb (GtkButton *button, AlmanahMainWindow *main_window)
-{
-	add_link_to_current_entry (main_window);
-}
-
-void
-mw_remove_button_clicked_cb (GtkButton *button, AlmanahMainWindow *main_window)
-{
-	remove_link_from_current_entry (main_window);
-}
-
 void
 mw_view_button_clicked_cb (GtkButton *button, AlmanahMainWindow *main_window)
 {
-	GList *links;
+	GList *events;
 	GtkTreeModel *model;
 
-	links = gtk_tree_selection_get_selected_rows (main_window->priv->links_selection, &model);
+	events = gtk_tree_selection_get_selected_rows (main_window->priv->events_selection, &model);
 
-	for (; links != NULL; links = links->next) {
-		AlmanahLink *link;
-		gchar *value, *value2, *type;
+	for (; events != NULL; events = events->next) {
+		AlmanahEvent *event;
 		GtkTreeIter iter;
 
-		gtk_tree_model_get_iter (model, &iter, (GtkTreePath*) links->data);
+		gtk_tree_model_get_iter (model, &iter, (GtkTreePath*) events->data);
 		gtk_tree_model_get (model, &iter,
-				    0, &type,
-				    1, &value,
-				    2, &value2,
+				    0, &event,
 				    -1);
 
-		link = almanah_link_new (type);
-		almanah_link_set_value (link, value);
-		almanah_link_set_value2 (link, value2);
+		/* NOTE: event types should display their own errors, so one won't be displayed here. */
+		almanah_event_view (event);
 
-		/* NOTE: Link types should display their own errors, so one won't be displayed here. */
-		almanah_link_view (link);
-
-		g_free (type);
-		g_free (value);
-		g_free (value2);
-		g_object_unref (link);
-
-		gtk_tree_path_free (links->data);
+		gtk_tree_path_free (events->data);
 	}
 
-	g_list_free (links);
+	g_list_free (events);
 }
+
+static void
+mw_definition_removed_cb (AlmanahStorageManager *storage_manager, const gchar *definition_text, AlmanahMainWindow *main_window)
+{
+	/* We need to remove any definition tags in the current entry which match @definition_text. It's
+	 * probably easier to just reload the current entry, though. */
+	mw_calendar_day_selected_cb (main_window->priv->calendar, main_window);
+}
+
+#ifdef ENABLE_SPELL_CHECKING
+gboolean
+almanah_main_window_enable_spell_checking (AlmanahMainWindow *self, GError **error)
+{
+	GtkSpell *gtkspell;
+	gchar *spelling_language;
+
+	spelling_language = gconf_client_get_string (almanah->gconf_client, "/apps/almanah/spelling_language", NULL);
+
+	/* Make sure it's either NULL or a proper locale specifier */
+	if (spelling_language != NULL && spelling_language[0] == '\0') {
+		g_free (spelling_language);
+		spelling_language = NULL;
+	}
+
+	gtkspell = gtkspell_new_attach (self->priv->entry_view, spelling_language, error);
+	g_free (spelling_language);
+
+	if (gtkspell == NULL)
+		return FALSE;
+	return TRUE;
+}
+
+void
+almanah_main_window_disable_spell_checking (AlmanahMainWindow *self)
+{
+	GtkSpell *gtkspell;
+
+	gtkspell = gtkspell_get_from_text_view (self->priv->entry_view);
+	if (gtkspell != NULL) {
+		gtkspell_detach (gtkspell);
+		g_object_unref (gtkspell);
+	}
+}
+#endif /* ENABLE_SPELL_CHECKING */
